@@ -17,7 +17,6 @@ email                : geiseri@msoe.edu
 
 #include "config.h"
 #include <qfile.h>
-#include <klocale.h>
 #include <qdatetime.h>
 #include <qtextstream.h>
 #include <kglobal.h>
@@ -31,6 +30,7 @@ email                : geiseri@msoe.edu
 #include <ktempfile.h>
 #include <unistd.h>
 #include <kapplication.h>
+#include <kpassivepopup.h>
 #include "weatherlib.h"
 
 #include "weatherlib.moc"
@@ -41,9 +41,10 @@ class WeatherLib::Data
 		Data()
 		{
 			theWeather = QString::null;
-			clouds = 0;
+			clouds = -1;
 			windMPH = 0;
 			tempF = 0;
+			heavy = false;
 			qsCoverList.clear();
 			qsCurrentList.clear();
 			qsDate = QDate::currentDate();
@@ -73,6 +74,7 @@ class WeatherLib::Data
 		int clouds;
 		int windMPH;
 		float tempF;
+		bool heavy;
 		QStringList qsCoverList;
 		QStringList qsCurrentList;
 		QDate qsDate;
@@ -120,7 +122,8 @@ void WeatherLib::Data::clear()
 	job = 0L;
 }
 
-WeatherLib::WeatherLib(QObject *parent, const char *name) : QObject (parent, name)
+WeatherLib::WeatherLib(QObject *parent, const char *name)
+	: QObject (parent, name)
 {
 	CoverRegExp    = QRegExp("^(FEW|SCT|BKN|OVC|SKC|CLR|CAVOK)([0-9]{3})?(?:TCU|CB)?$");
 	CurrentRegExp  = QRegExp("^(\\+|-|VC)?([A-Z]{2,4})$");
@@ -141,7 +144,8 @@ WeatherLib::~WeatherLib()
 bool WeatherLib::parseCover(QString s, Data *d){
 	if (CoverRegExp.search(s) > -1)
 	{
-		kdDebug(12004) << "Cover: " << CoverRegExp.capturedTexts().join("-") << endl;
+		kdDebug(12004) << "Cover: " << CoverRegExp.capturedTexts().join("-")
+			<< endl;
 
 		QString sCode = CoverRegExp.cap(1);
 		float fFeet = CoverRegExp.cap(2).toFloat();
@@ -152,47 +156,48 @@ bool WeatherLib::parseCover(QString s, Data *d){
 		{
 			fFeet = fFeet * 0.3048;  // Check this!
 		}
-		
+
 		if (fFeet >= 10)
 		{
 			fFeet /= 10;
 			if (KGlobal::locale()->measureSystem() == KLocale::Metric)
-				sClouds = QString("%1 %2").arg(fFeet,0,'f',1).arg(i18n(" kilometers"));
+				sClouds = i18n("%1 kilometers").arg(fFeet,0,'f',1);
 			else
-				sClouds = QString("%1 %2").arg(fFeet,0,'f',1).arg(i18n(" thousand feet"));
+				sClouds = i18n("%1 thousand feet").arg(fFeet,0,'f',1);
 		}
 		else
 		{
 			fFeet *= 100;
 			if (KGlobal::locale()->measureSystem() == KLocale::Metric)
-				sClouds = QString("%1 %2").arg(fFeet,0,'f',0).arg(i18n("meters"));
+				sClouds = i18n("%1 meters").arg(fFeet,0,'f',0);
 			else
-				sClouds = QString("%1 %2").arg(fFeet).arg(i18n("feet"));
+				sClouds = i18n("%1 feet").arg(fFeet);
 		}
 
 		if (sCode == "FEW")
 		{
-			skycondition = i18n( "Few clouds at %1." ).arg(sClouds);
+			skycondition = i18n( "Few clouds at %1" ).arg(sClouds);
 			d->clouds += 2;
 		}
 		else if (sCode == "SCT")
 		{
-			skycondition = i18n( "Scattered clouds at %1." ).arg(sClouds);
+			skycondition = i18n( "Scattered clouds at %1" ).arg(sClouds);
 			d->clouds += 4;
 		}
 		else if (sCode == "BKN")
 		{
-			skycondition = i18n( "Broken clouds at %1." ).arg(sClouds);
+			skycondition = i18n( "Broken clouds at %1" ).arg(sClouds);
 			d->clouds += 8;
 		}
 		else if (sCode == "OVC")
 		{
-			skycondition = i18n( "Overcast clouds at %1." ).arg(sClouds);
+			skycondition = i18n( "Overcast clouds at %1" ).arg(sClouds);
 			d->clouds += 64;
 		}
 		else if ((sCode == "CLR") || (sCode == "SKC") || (sCode == "CAVOK"))
 		{
-			skycondition = i18n("Clear skies.");
+			skycondition = i18n("Clear skies");
+			d->clouds = 0;
 		}
 
 		kdDebug(12004) << "*** Clouds: " << d->clouds << endl;
@@ -215,9 +220,15 @@ bool WeatherLib::parseCurrent(QString s, Data *d){
 
 		// Decode the intensity
 		if (sIntensity == "+")
+		{
 			currentWeather = i18n("Heavy");
+			d->heavy = true;
+		}
 		else if (sIntensity == "-")
+		{
 			currentWeather = i18n("Light");
+			d->heavy = false;
+		}
 
 		// Decode the descriptor
 		if (sCode.contains("MI"))
@@ -294,12 +305,20 @@ bool WeatherLib::parseCurrent(QString s, Data *d){
 		else if (sCode.contains("BR"))
 		{
 			currentWeather += i18n(" Mist");
-			d->theWeather = "mist";
+                       // Mist has lower priority than say rain or snow
+                       if ( d->theWeather.isEmpty() )
+                       {
+                               d->theWeather = "mist";
+                       }
 		}
 		else if (sCode.contains("FG"))
 		{
 			currentWeather += i18n(" Fog");
-			d->theWeather = "fog";
+                       // Fog has lower priority than say rain or snow
+                       if ( d->theWeather.isEmpty() )
+                       {
+                               d->theWeather = "fog";
+                       }
 		}
 		else if (sCode.contains("FU"))
 			currentWeather += i18n(" Smoke");
@@ -342,7 +361,8 @@ bool WeatherLib::parseTemperature(QString s, Data *d){
 	if (TempRegExp.search(s) > -1)
 	{
 		QString unit;
-		kdDebug(12004) << "Temp: " << TempRegExp.capturedTexts().join("-") << endl;
+		kdDebug(12004) << "Temp: " << TempRegExp.capturedTexts().join("-")
+			<< endl;
 
 		float fTemp = TempRegExp.cap(2).toFloat();
 		if (TempRegExp.cap(1) == "M")
@@ -361,10 +381,13 @@ bool WeatherLib::parseTemperature(QString s, Data *d){
 		if (d->tempF >= 80)
 		{
 			#define SQR(a)  ((a)*(a))
-			fHeatIndex = -42.379 + (2.04901523*d->tempF) + (10.14333127*fRelHumidity)
-				- (0.22475541*d->tempF*fRelHumidity) - (0.00683783*SQR(d->tempF))
-				- (0.05481717*SQR(fRelHumidity)) + (0.00122874*SQR(d->tempF)*fRelHumidity)
-				+ (0.00085282*d->tempF*SQR(fRelHumidity)) 
+			fHeatIndex = -42.379 + (2.04901523*d->tempF)
+				+ (10.14333127*fRelHumidity)
+				- (0.22475541*d->tempF*fRelHumidity)
+				- (0.00683783*SQR(d->tempF))
+				- (0.05481717*SQR(fRelHumidity))
+				+ (0.00122874*SQR(d->tempF)*fRelHumidity)
+				+ (0.00085282*d->tempF*SQR(fRelHumidity))
 				- (0.00000199*SQR(d->tempF)*SQR(fRelHumidity));
 
 			if ( fHeatIndex <= d->tempF )
@@ -401,17 +424,18 @@ bool WeatherLib::parseTemperature(QString s, Data *d){
 bool WeatherLib::parseDate(QString s, Data *d){
 	if (DateRegExp.search(s) > -1)
 	{
-		kdDebug(12004) << "Date: " << DateRegExp.capturedTexts().join("-") << endl;
+		kdDebug(12004) << "Date: " << DateRegExp.capturedTexts().join("-")
+			<< endl;
 		QString dateString = DateRegExp.cap(1);
 		QString day, month, year;
 
 		day = dateString.mid(8,2);
 		month = dateString.mid(5,2);
 		year = dateString.mid(0,4);
-		
+
 		QDate theDate(year.toInt(), month.toInt(), day.toInt());
 
-		
+
 		d->qsDate = theDate;
 		return true;
 	}
@@ -422,7 +446,8 @@ bool WeatherLib::parseDate(QString s, Data *d){
 bool WeatherLib::parseTime(QString s, Data *d){
 	if (TimeRegExp.search(s) > -1)
 	{
-		kdDebug(12004) << "Time: " << TimeRegExp.capturedTexts().join("-") << endl;
+		kdDebug(12004) << "Time: " << TimeRegExp.capturedTexts().join("-")
+			<< endl;
 
 		QString hour, minute, dateString;
 
@@ -443,7 +468,8 @@ bool WeatherLib::parseVisibility( QString s, Data *d){
 	{
 		float fVisibility = VisRegExp.cap(1).toFloat();
 
-		kdDebug(12004) << "Visibility: " << VisRegExp.capturedTexts().join("-") << endl;
+		kdDebug(12004) << "Visibility: " << VisRegExp.capturedTexts().join("-")
+			<< endl;
 
 		if (KGlobal::locale()->measureSystem() == KLocale::Metric)
 		{
@@ -468,7 +494,8 @@ bool WeatherLib::parsePressure( QString s, Data *d){
 		QString type = PressRegExp.cap(1);
 		float fPressure = PressRegExp.cap(2).toFloat();
 
-		kdDebug(12004) << "Pressure: " << PressRegExp.capturedTexts().join("-") << endl;
+		kdDebug(12004) << "Pressure: " << PressRegExp.capturedTexts().join("-")
+			<< endl;
 
 		if (KGlobal::locale()->measureSystem() == KLocale::Metric)
 		{
@@ -501,7 +528,8 @@ bool WeatherLib::parseWindSpeed(QString s, Data *d)
 		int iGustSpeed = WindRegExp.cap(3).toInt();
 		QString sWindUnit = WindRegExp.cap(4);
 
-		kdDebug(12004) << "Wind: " << WindRegExp.capturedTexts().join("-") << endl;
+		kdDebug(12004) << "Wind: " << WindRegExp.capturedTexts().join("-")
+			<< endl;
 
 		if (KGlobal::locale()->measureSystem() == KLocale::Metric)
 		{
@@ -543,11 +571,13 @@ bool WeatherLib::parseWindSpeed(QString s, Data *d)
 		d->qsWindSpeed += sWindUnit;
 
 		if (iGustSpeed > 0)
-			d->qsCurrentList << i18n("Wind gusts up to %1 %2").arg(iGustSpeed).arg(sWindUnit);
+			d->qsCurrentList << i18n("Wind gusts up to %1 %2")
+				.arg(iGustSpeed).arg(sWindUnit);
 
 		if ((WindRegExp.cap(1) != "VRB") && (iWindSpeed > 0))
 		{
-			if ((iDirection >=0 && iDirection <= 11) || ( iDirection >= 349 && iDirection <= 360))
+			if ((iDirection >=0 && iDirection <= 11)
+					|| ( iDirection >= 349 && iDirection <= 360))
 				d->qsWindDirection = i18n("N");
 			else if (iDirection >= 12 && iDirection <= 32)
 				d->qsWindDirection = i18n("NNE");
@@ -586,6 +616,9 @@ bool WeatherLib::parseWindSpeed(QString s, Data *d)
 }
 
 void WeatherLib::calcCurrentIcon(Data *d){
+       // Default to overcast clouds
+       if ( d->clouds == -1 )
+               d->clouds = 64;
 	if (d->theWeather.isEmpty())
 	{
 		if (d->clouds == 0)
@@ -596,13 +629,16 @@ void WeatherLib::calcCurrentIcon(Data *d){
 			d->theWeather = "cloudy2";
 		else if ( d->clouds > 4 && d->clouds <= 8)
 			d->theWeather = "cloudy3";
-		else if ( d->clouds > 8 && d->clouds < 64)
+		else if ( d->clouds > 8 && d->clouds < 63)
 			d->theWeather = "cloudy4";
 		else
 			d->theWeather = "cloudy5";
 	}
 	else if (d->theWeather == "tstorm")
 	{
+		if ( d->heavy )
+			d->clouds = 30;
+
 		if (d->clouds >= 0 && d->clouds <= 10)
 			d->theWeather = "tstorm1";
 		else if ( d->clouds > 10 && d->clouds <= 20)
@@ -612,6 +648,9 @@ void WeatherLib::calcCurrentIcon(Data *d){
 	}
 	else if (d->theWeather == "shower")
 	{
+		if ( d->heavy )
+			d->clouds = 30;
+
 		if (d->clouds >= 0 && d->clouds <= 10)
 			d->theWeather = "shower1";
 		else if ( d->clouds > 10 && d->clouds <= 20)
@@ -621,6 +660,9 @@ void WeatherLib::calcCurrentIcon(Data *d){
 	}
 	else if (d->theWeather == "snow")
 	{
+		if ( d->heavy )
+			d->clouds = 30;
+
 		if (d->clouds >= 0 && d->clouds <= 8)
 			d->theWeather = "snow1";
 		else if ( d->clouds > 8 && d->clouds <= 16)
@@ -631,11 +673,13 @@ void WeatherLib::calcCurrentIcon(Data *d){
 			d->theWeather = "snow5";
 	}
 
-	kdDebug(12004) << "Clouds: " << d->clouds << ", Icon: " << d->theWeather << endl;
+	kdDebug(12004) << "Clouds: " << d->clouds << ", Icon: "
+		<< d->theWeather << endl;
 }
 
 void WeatherLib::calcWindChill(Data *d){
-	float windChill = 35.74 + (0.6215*d->tempF) - (35.75*::pow(d->windMPH, 0.16))
+	float windChill = 35.74 + (0.6215*d->tempF)
+		- (35.75*::pow(d->windMPH, 0.16))
 		+ (0.4275*d->tempF*::pow(d->windMPH, 0.16));
 
 	kdDebug(12004) << "Wind Chill: " << windChill << endl;
@@ -664,7 +708,8 @@ void WeatherLib::processData(QString metar, Data *d){
 	QStringList dataList = QStringList::split(QRegExp("\\s+"), metar);
 	bool found = false;
 
-	for (QStringList::ConstIterator it = dataList.begin(); it != dataList.end(); ++it)
+	for (QStringList::ConstIterator it = dataList.begin();
+			it != dataList.end(); ++it)
 	{
 		// Abort if ReMarK code
 		if (*it == "RMK")
@@ -704,8 +749,19 @@ void WeatherLib::slotCopyDone(KIO::Job* job)
 {
 	kdDebug(12004) << "Copy done..." << endl;
 	if( job->error())
-		job->showErrorDialog(0L);
-
+	{
+		kdDebug(12004) << "Error code: " << job->error() << endl;
+		//job->showErrorDialog(0L);
+		if(job->error() == KIO::ERR_COULD_NOT_CONNECT ||
+			job->error() == KIO::ERR_UNKNOWN_HOST)
+		{
+			hostDown= true;
+			// should not bother after this point
+			QPixmap pix();
+			KPassivePopup::message( i18n("KWeather Error!"),
+				i18n("The network is unavailable for use."),  0L,"error" );
+		}
+	}
 	// Find the job
 	QDictIterator<Data> it( data );
 	Data *d = 0L;
@@ -714,44 +770,84 @@ void WeatherLib::slotCopyDone(KIO::Job* job)
 		kdDebug(12004) << "Searching for job..." << endl;
 		if(it.current()->job == job)
 		{
-			
+
 			d = it.current();
-			QFile f(d->target->name());
-			if ( f.open(IO_ReadOnly) )
-			{
-				QTextStream t(&f);
-				QString s;
-				while ( !t.eof() ) 
-				{ 
-					s += " " + t.readLine();
-					kdDebug( 12004 ) << s << endl;
-				}
-				processData(s, d);
-				d->age = QDateTime::currentDateTime().addSecs(3000);
-			}
-			else
-			{
-				// File error
-				kdDebug( 12004 ) << "File read error..." << endl;
-			}
-			f.close();
 			d->downloading = false;
-			delete d->target;
-			emit  fileUpdate(QString(d->reportLocation));
-			d->updated = true;
-			
-			if( job->error())
+			if( !job->error() )
+			{
+				QFile f(d->target->name());
+				if ( f.open(IO_ReadOnly) )
+				{
+					QTextStream t(&f);
+					QString s;
+					while ( !t.eof() )
+					{
+						s += " " + t.readLine();
+						kdDebug( 12004 ) << s << endl;
+					}
+					processData(s, d);
+					d->age = QDateTime::currentDateTime().addSecs(3000);
+					emit  fileUpdate(QString(d->reportLocation));
+					d->updated = true;
+				}
+				else
+				{
+					// File error
+					kdDebug( 12004 ) << "File read error..." << endl;
+					KPassivePopup::message( i18n("KWeather Error!"),
+				i18n("Could not read the temp file."),  0L,"error" );
+					d->updated = false;
+				}
+				f.close();
+				delete d->target;
+
+			}
+			else if( job->error()  == KIO::ERR_DOES_NOT_EXIST)
 			{
 				data.remove(d->reportLocation);
 				kdDebug( 12004 ) << "Bad station data so i am going to remove it" << endl;
+				KPassivePopup::message( i18n("KWeather Error!"),
+				i18n("The requested station does not exist."),  0L,"error" );
 			}
+			else if(job->error() == KIO::ERR_COULD_NOT_CONNECT ||
+				job->error() == KIO::ERR_UNKNOWN_HOST)
+			{
+				kdDebug( 12004 ) << "Offline now..." << endl;
+				d->updated = false;
+				d->theWeather = "dunno";
+				d->clouds = 0;
+				d->windMPH = 0;
+				d->tempF = 0;
+				d->qsCoverList.clear();
+				d->qsCurrentList.clear();
+				d->qsCurrentList.append(i18n("The network is currently offline..."));
+				d->qsCurrentList.append(i18n("Please update later."));
+				d->qsDate = QDate::currentDate();
+				d->qsPressure = "XX";
+				d->qsTemperature = "XX";
+				d->qsDewPoint = "XX";
+				d->qsRelHumidity = "XX";
+				d->qsTime = QTime::currentTime();
+				d->qsVisibility = "XX";
+				d->qsWindSpeed = "XX";
+				d->qsWindChill = "XX";
+				d->qsHeatIndex = "XX";
+				d->qsWindDirection = "XX";
+				emit  fileUpdate(QString(d->reportLocation));
+				delete d->target;
+			}
+			else
+			{
+				kdDebug( 12004 ) << "Duh?..." << endl;
+			}
+
 		}
 	}
 }
 
 void WeatherLib::getData(Data *d)
 {
-	if(!d->downloading)
+	if(!d->downloading && !hostDown)
 	{
 		d->updated = false;
 		QString u = "http://weather.noaa.gov/pub/data/observations/metar/stations/";
@@ -764,9 +860,11 @@ void WeatherLib::getData(Data *d)
 		KURL local(d->target->name());
 
 		d->job = KIO::file_copy( url, local, -1, true, false, false);
-		connect( d->job, SIGNAL( result( KIO::Job *)), this, SLOT(slotCopyDone(KIO::Job *)));
+		connect( d->job, SIGNAL( result( KIO::Job *)),
+			SLOT(slotCopyDone(KIO::Job *)));
 		d->downloading = true;
-		kdDebug( 12004 ) << "Coping " << url.prettyURL() << " to " << local.prettyURL() << endl;
+		kdDebug( 12004 ) << "Copying " << url.prettyURL() << " to "
+			<< local.prettyURL() << endl;
 		emit fileUpdateing(QString(d->reportLocation));
 	}
 }
@@ -780,12 +878,13 @@ WeatherLib::Data* WeatherLib::findData(QString stationID)
 		d->reportLocation = stationID;
 		data.insert(stationID, d);
 		getData(d);
-		while(!d->updated)
+/*		while(!d->updated && !hostDown)
 		{
 			usleep(250);
 			//kdDebug( 12004 ) << "spin..." << endl;
 			kapp->processEvents();
 		}
+*/
 	}
 
 	return d;
@@ -834,10 +933,9 @@ QString WeatherLib::currentIcon(QString stationID){
 
 QString WeatherLib::date(QString stationID){
 	Data *d = findData(stationID);
-	
+
 	QDateTime gmtDateTime(d->qsDate, d->qsTime);
 	QDateTime localDateTime = gmtDateTime.addSecs(KRFCDate::localUTCOffset() * 60);
-	//KLocale *loc = KGlobal::locale();
 	return KGlobal::locale()->formatDateTime(localDateTime, false, false);
 }
 
@@ -861,14 +959,16 @@ QStringList WeatherLib::weather(QString stationID){
 
 void WeatherLib::update(QString stationID)
 {
+	// refresh hostdown just to see
+	hostDown = false;
 	// Only grab new data if its more than 50 minutes old
 	Data *d = findData(stationID);
-		
+
 	QDateTime timeout = QDateTime::currentDateTime();
 
 	kdDebug () << "Current Time: " << KGlobal::locale()->formatDateTime(timeout, false, false) <<
 			" Update at: " << KGlobal::locale()->formatDateTime(d->age, false, false) << endl;
-	if( timeout >= d->age)
+	if( timeout >= d->age || !d->updated)
 		getData(d);
 	else
 		 emit  fileUpdate(QString(d->reportLocation));
@@ -888,18 +988,20 @@ void WeatherLib::blockingUpdate(QString stationID)
 {
 	// Only grab new data if its more than 50 minutes old
 	Data *d = findData(stationID);
-		
+
 	QDateTime timeout = QDateTime::currentDateTime();
 
 	kdDebug () << "Current Time: " << KGlobal::locale()->formatDateTime(timeout, false, false) <<
 			" Update at: " << KGlobal::locale()->formatDateTime(d->age, false, false) << endl;
+	int giveup = 0;
 	if( timeout >= d->age)
 		getData(d);
-	while(!d->updated)
+	while(!d->updated && !hostDown && giveup < 140) // Timeout after 30 seconds...
 	{
 		usleep(250);
 		//kdDebug( 12004 ) << "spin..." << endl;
 		kapp->processEvents();
+		++giveup;
 	}
 }
 
